/*******************************************************************************
 * Copyright (c) 2008 Sybase, Inc.
 * 
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License v1.0 which
 * accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Sybase - initial API and implementation
 ******************************************************************************/
package org.eclipse.datatools.enablement.sybase.ui;

import java.util.Properties;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.eclipse.jface.dialogs.DialogPage;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;


import org.eclipse.datatools.connectivity.ConnectionProfileConstants;
import org.eclipse.datatools.connectivity.IConnectionProfile;
import org.eclipse.datatools.connectivity.db.generic.IDBConnectionProfileConstants;
import org.eclipse.datatools.connectivity.db.generic.IDBDriverDefinitionConstants;
import org.eclipse.datatools.connectivity.drivers.DriverInstance;
import org.eclipse.datatools.connectivity.drivers.IDriverMgmtConstants;
import org.eclipse.datatools.connectivity.internal.ui.DelimitedStringList;
import org.eclipse.datatools.connectivity.internal.ui.DriverListCombo;
import org.eclipse.datatools.enablement.sybase.ISybaseJDBCConnectionProfileConstants;

/**
 * Tabbed control to handle gathering JDBC profile info.
 * Tabs for connection info, filters (catalog, schema, SP)
 * and optional properties (i.e. DYNAMICPREPARE=true).
 * @author brianf
 *
 */
public class JDBCProfileTabs extends DialogPage {

	// simple constant
	protected static final String EMPTY_STRING = ""; //$NON-NLS-1$

	// UI pieces - tabs
	protected Composite rootComposite;
	protected TabFolder tabfolder;
	protected TabItem connTab;
	protected Composite connTabComposite;
	protected TabItem filtersTab;
	protected Composite filtersTabComposite;
	protected TabItem otherTab;
	protected Composite otherTabComposite;

	// UI pieces - text
	protected  Text mDBUIDText;
	protected  Text mDBPWDText;
	protected  Text mURLText;
	
	protected  Text mCatalogText;
	protected  Text mSchemaText;
	protected  Text mProcNameText;

	// UI pieces - driver-related
	protected  DriverListCombo combo = new DriverListCombo();
	protected  DriverInstance mDriverInstance;
	
	// For "other" properties
	//cr400484
	protected  DelimitedStringList mDBConnProps;
	
	// listeners
	protected ChangeListener comboListener;
	private ListenerList changeListeners;
	
	// reference to profile
	protected  IConnectionProfile mProfile;
	
	// provider ID
	protected String mProviderID = null;
	
	// tab flags
	protected boolean mShowFilterTab = true;
	protected boolean mShowOptionalTab = true;
	
	//cr381632 old selected drive name
	private String _preSelectedDriveName = null;
	
	private boolean _resetPwdFromDriver = true;

	/**
	 * Constructor
	 */
	public JDBCProfileTabs() {
		this.changeListeners = new ListenerList();
	}
	
	public boolean getResetPwdFromDriver() {
		return this._resetPwdFromDriver;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		this.rootComposite = parent;
		this.tabfolder = new TabFolder(parent, SWT.TOP
				| SWT.MULTI  );
		GridData gdata = new GridData(GridData.FILL_BOTH);
		gdata.grabExcessVerticalSpace = true;
		gdata.grabExcessHorizontalSpace = true;
		gdata.horizontalSpan = 2;
		this.tabfolder.setLayoutData(gdata);
		
		// Connection tab
		this.connTab = new TabItem(this.tabfolder, SWT.NONE);
		this.connTab.setText(JDBCProfileMessages.getString("JDBCProfileTabs.tabs.label.connection")); //$NON-NLS-1$
		createConnectionTab ( this.tabfolder );
		this.connTab.setControl( this.connTabComposite );
		
		// Filters tab
		if (this.mShowFilterTab) {
			this.filtersTab = new TabItem(this.tabfolder, SWT.NONE);
			this.filtersTab.setText(JDBCProfileMessages.getString("JDBCProfileTabs.tabs.label.filters")); //$NON-NLS-1$
			createFilterTab( this.tabfolder );
			this.filtersTab.setControl( this.filtersTabComposite );
		}

		// Optional props tab
		if (this.mShowOptionalTab) {
			this.otherTab = new TabItem(this.tabfolder, SWT.NONE);
			this.otherTab.setText(JDBCProfileMessages.getString("JDBCProfileTabs.tabs.label.other")); //$NON-NLS-1$
			createOtherTab( this.tabfolder );
			this.otherTab.setControl( this.otherTabComposite );
		}

		this.tabfolder.setSelection(0);
		
		initControls();
	}

	/**
	 * @param profile
	 */
	public void setConnectionProfile ( IConnectionProfile profile ) {
		this.mProfile = profile;
		initControls();
	}
	
	/**
	 * @return
	 */
	public IConnectionProfile getConnectionProfile () {
		return this.mProfile;
	}
	
	/* (non-Javadoc)
	 * @see com.sybase.stf.common.ui.dialogs.IDialogPage#performOk()
	 */
	public boolean performOk() {
		return false;
	}
	
	/**
	 * @param providerID
	 */
	public void setProviderID ( String providerID ) {
		this.mProviderID = providerID;
	}
	
	/**
	 * @return
	 */
	public String getProviderID () {
		return this.mProviderID;
	}
	
	/**
	 * @param parent
	 */
	protected void createOtherTab ( TabFolder parent ) {
		otherTabComposite = new Composite ( parent, SWT.NONE );
		
		GridLayout layout = new GridLayout(2,false);
		otherTabComposite.setLayout(layout);
	
		this.mDBConnProps = new DelimitedStringList(otherTabComposite, SWT.NONE );


		GridData gdata = new GridData(GridData.FILL_HORIZONTAL);
		gdata.horizontalSpan = 2;
		//cr400484
		//this.mDBConnProps.setLayoutData(gdata);
		((DelimitedStringList)this.mDBConnProps).setLayoutData(gdata);
        
		this.mDBConnProps.addChangeListener( new ChangeListener() {

			public void stateChanged(ChangeEvent arg0) {
				setErrorMessage(null);
				if (JDBCProfileTabs.this.mDBConnProps.getWarning() != null) {
					setErrorMessage(JDBCProfileTabs.this.mDBConnProps.getWarning());
				}
			}
			
		});
	}
	
	/**
	 * @param parent
	 */
	protected void createFilterTab ( TabFolder parent ) {
		filtersTabComposite = new Composite(parent, SWT.NONE);

		GridLayout layout = new GridLayout(2,false);
		filtersTabComposite.setLayout(layout);

		this.mCatalogText = (Text) createLabelTextPair (filtersTabComposite, 
				JDBCProfileMessages.getString("JDBCPropertyWizardPage.catalog.label"),  //$NON-NLS-1$
				this.mCatalogText, SWT.BORDER, GridData.FILL_HORIZONTAL);
		this.mSchemaText = (Text) createLabelTextPair (filtersTabComposite, 
				JDBCProfileMessages.getString("JDBCPropertyWizardPage.schema.label"),  //$NON-NLS-1$
				this.mSchemaText, SWT.BORDER, GridData.FILL_HORIZONTAL);
		this.mProcNameText = (Text) createLabelTextPair (filtersTabComposite, 
				JDBCProfileMessages.getString("JDBCPropertyWizardPage.procedureName.label"),  //$NON-NLS-1$
				this.mProcNameText, SWT.BORDER, GridData.FILL_HORIZONTAL);
	}
	
	/**
	 * @param parent
	 */
	protected void createConnectionTab ( TabFolder parent ) {
		connTabComposite = new Composite(parent, SWT.NONE);
		
		GridLayout layout = new GridLayout(2,false);
		connTabComposite.setLayout(layout);
		
		this.combo.setLabelText(JDBCProfileMessages.getString("JDBCPropertyWizardPage.driverCombo.label")); //$NON-NLS-1$
		this.combo.setCategory("com.sybase.stf.servers.jdbc.sybase"); //$NON-NLS-1$
		this.combo.setNullDriverIsValid(false);
		this.combo.createContents(this.connTabComposite);
		if (this.combo.getErrorMessage() != null) {
			setErrorMessage(this.combo.getErrorMessage());
		}

		this.mURLText = (Text) createLabelTextPair (connTabComposite, 
				JDBCProfileMessages.getString("JDBCPropertyWizardPage.url.label"),  //$NON-NLS-1$
				this.mURLText, SWT.BORDER, SWT.NULL);
		this.mURLText.addModifyListener( new ModifyListener() {

			public void modifyText(ModifyEvent e) {
				fireChangedEvent(JDBCProfileTabs.this);
			}
		});

		this.mDBUIDText = (Text) createLabelTextPair (connTabComposite, 
				JDBCProfileMessages.getString("JDBCPropertyWizardPage.userName.label"),  //$NON-NLS-1$ 
				this.mDBUIDText, SWT.BORDER, GridData.FILL_HORIZONTAL); //$NON-NLS-1$
		this.mDBUIDText.addModifyListener( new ModifyListener() {

			public void modifyText(ModifyEvent e) {
				fireChangedEvent(JDBCProfileTabs.this);
			}
		});

		this.mDBPWDText = (Text) createLabelTextPair (connTabComposite, 
				JDBCProfileMessages.getString("JDBCPropertyWizardPage.password.label"),  //$NON-NLS-1$, 
				this.mDBPWDText, SWT.BORDER, GridData.FILL_HORIZONTAL); //$NON-NLS-1$
		this.mDBPWDText.setEchoChar('*');
		this.mDBPWDText.addModifyListener( new ModifyListener() {

			public void modifyText(ModifyEvent e) {
				fireChangedEvent(JDBCProfileTabs.this);
			}
		});

		comboListener = new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				JDBCProfileTabs.this.driverStateChanged();
			}
		};
		this.combo.addChangeListener( comboListener );
	}
	
	/**
	 * Update profile info when the driver selection changes 
	 */
	protected void driverStateChanged() {
		this.mDriverInstance = this.combo.getSelectedDriverInstance();
		setErrorMessage(null);
		if (this.combo.getErrorMessage() != null) {
			setErrorMessage(this.combo.getErrorMessage());
		}
		else {
			if (this.mURLText != null)
			{	//cr381632 reset URL only at driver switched
				if (! this.combo.getSelectedDriverName().equals(_preSelectedDriveName))
						this.mURLText.setText(getDriverURL());
			}
		}
			_preSelectedDriveName = this.combo.getSelectedDriverName();

		
		String username =  this.getNamedPropertyFromDriverInstance("User ID"); //$NON-NLS-1$
		if (username == null || username.trim().length() == 0) username = "";//$NON-NLS-1$
		String oldUsername = this.mDBUIDText.getText();
		if (oldUsername == null || oldUsername.trim().length() == 0) oldUsername = "";//$NON-NLS-1$
		this.mDBUIDText.setText(username);

		String password =  this.getNamedPropertyFromDriverInstance("Password"); //$NON-NLS-1$
		if (password == null || password.trim().length() == 0) password = "";//$NON-NLS-1$
		String oldPassword = this.mDBPWDText.getText();
		if (oldPassword == null || oldPassword.trim().length() == 0) oldPassword = "";//$NON-NLS-1$
		if (this._resetPwdFromDriver)
			this.mDBPWDText.setText(password);

		if (this.mShowFilterTab ) {
			String catalog = this.getNamedPropertyFromDriverInstance("catalog"); //$NON-NLS-1$
			if (catalog == null) catalog = "%"; //$NON-NLS-1$
			if (!catalog.startsWith("[")) //$NON-NLS-1$
				this.mCatalogText.setText(catalog);
			
			String schema = this.getNamedPropertyFromDriverInstance("schema"); //$NON-NLS-1$
			if (schema == null || schema.trim().length() == 0) schema = "%"; //$NON-NLS-1$
			if (!schema.startsWith("[")) //$NON-NLS-1$
				this.mSchemaText.setText(schema);
	
			String procName = this.getNamedPropertyFromDriverInstance("procname"); //$NON-NLS-1$
			if (procName == null || procName.trim().length() == 0) procName = "%"; //$NON-NLS-1$
			if (!procName.startsWith("[")) //$NON-NLS-1$
				this.mProcNameText.setText(procName);
		}
		
		if (this.mShowOptionalTab) {
			String connProps = this.getNamedPropertyFromDriverInstance("connectionProps"); //$NON-NLS-1$
			if (connProps == null || connProps.trim().length() == 0) connProps = ""; //$NON-NLS-1$
			if (!connProps.startsWith("[")) //$NON-NLS-1$
				this.mDBConnProps.setSelection(connProps);
		}

		fireChangedEvent(this);
	}

	/**
	 * @param propertyName
	 * @return
	 */
	public String getNamedPropertyFromDriverInstance( String propertyName ) {
		String returnStr = ""; //$NON-NLS-1$
		if (this.mDriverInstance != null && this.mDriverInstance.getNamedProperty(propertyName) != null) {
			returnStr = this.mDriverInstance.getNamedProperty(propertyName);
		}
		return returnStr;
	}

	/**
	 * @param parent
	 * @param labelText
	 * @param ctl
	 * @param style
	 * @param gData
	 * @return
	 */
	protected Control createLabelTextPair (Composite parent, String labelText, Control ctl, int style, int gData) {
		Label label = new Label(parent,SWT.NULL);
		label.setLayoutData(new GridData(SWT.FILL, SWT.NULL, false, false));
		label.setText(labelText);
		
		ctl = new Text(parent, style);
		ctl.setLayoutData(new GridData(SWT.FILL, SWT.NULL, true, false));
		
		return ctl;
	}

	/**
	 * @return
	 */
	public String getDriverClass() {
		if (this.mDriverInstance != null)
			return this.mDriverInstance.getNamedProperty("Driver Class"); //$NON-NLS-1$
		return ""; //$NON-NLS-1$
	}

	/**
	 * @return
	 */
	public String getDriverURL() {
		if (this.mDriverInstance != null)
			return this.mDriverInstance.getNamedProperty("Connection URL"); //$NON-NLS-1$
		return ""; //$NON-NLS-1$
	}

	/**
	 * Get the user name
	 * @return
	 */
	public String getDBUID() {
		return this.mDBUIDText.getText();
	}

	/**
	 * Get the password
	 * @return
	 */
	public String getDBPWD() {
		return this.mDBPWDText.getText();
	}

	/**
	 * Get the optional connection properties
	 * @return
	 */
	public String getDBConnProps() {
		if (this.mDBConnProps != null)
			return this.mDBConnProps.getSelection();
		return ""; //$NON-NLS-1$
	}
	
	/**
	 * Get the URL for the CP
	 * @return
	 */
	public String getURL() {
		return this.mURLText.getText();
	}

	/**
	 * Get the driver name
	 * @return
	 */
	public String getDriverName() {
		return this.mDriverInstance.getName();
	}
	
	/**
	 * Get the driver ID
	 * @return
	 */
	public String getDriverID() {
		return this.mDriverInstance.getId();
	}

	/**
	 * Get the driver ID
	 * @return
	 */
	public String getDriverDefnId() {
		return this.mDriverInstance.getNamedProperty(IDriverMgmtConstants.PROP_DEFN_TYPE);
	}

	/**
	 * Get the catalog
	 * @return
	 */
	public String getCatalog() {
		if (this.mCatalogText != null)
			return this.mCatalogText.getText();
		return "%"; //$NON-NLS-1$
	}
	/**
	 * @return
	 */
	public String getSchema() {
		if (this.mSchemaText != null)
			return this.mSchemaText.getText();
		return "%"; //$NON-NLS-1$
	}
	/**
	 * @return
	 */
	public String getProcName() {
		if (this.mProcNameText != null)
			return this.mProcNameText.getText();
		return "%"; //$NON-NLS-1$
	}

	/**
	 * Get the jar list from the driver instance.
	 * @return
	 */
	public String getDriverJarList() {
		return this.mDriverInstance.getJarList();
	}

	/**
     * 
     */
    protected void initControls() {
        IConnectionProfile profile = getConnectionProfile();
        if (profile != null) {
	        String driverID = profile.getBaseProperties().getProperty(ConnectionProfileConstants.PROP_DRIVER_DEFINITION_ID);    
	        if (driverID != null) {
	        	this.combo.setSelectionToID(driverID);
	        	this.combo.getCombo().redraw();
	        }
	
	        String connectionProps = profile.getBaseProperties().getProperty(IDBConnectionProfileConstants.CONNECTION_PROPERTIES_PROP_ID);
	        if (connectionProps != null && this.mDBConnProps != null) {
	        	this.mDBConnProps.setSelection(connectionProps);
	        }
	
	        String databasePwd = profile.getBaseProperties().getProperty(IDBDriverDefinitionConstants.PASSWORD_PROP_ID); //$NON-NLS-1$
	        if (databasePwd != null) {
	        	this.mDBPWDText.setText(databasePwd);
	        }
	
	        String databaseUid = profile.getBaseProperties().getProperty(IDBDriverDefinitionConstants.USERNAME_PROP_ID); //$NON-NLS-1$
	        if (databaseUid != null) {
	        	this.mDBUIDText.setText(databaseUid);
	        }
	
	        String urlText = profile.getBaseProperties().getProperty(IDBDriverDefinitionConstants.URL_PROP_ID); //$NON-NLS-1$
	        if (urlText != null && this.mURLText != null) {
	        	this.mURLText.setText(urlText);
	        }
	
	        String catalog = profile.getBaseProperties().getProperty(ISybaseJDBCConnectionProfileConstants.PROP_CATALOG); //$NON-NLS-1$
	        if (catalog != null && this.mCatalogText != null) {
	        	this.mCatalogText.setText(catalog);
	        }
	
	        String schema = profile.getBaseProperties().getProperty(ISybaseJDBCConnectionProfileConstants.PROP_SCHEMA); //$NON-NLS-1$
	        if (schema != null && this.mSchemaText != null) {
	        	this.mSchemaText.setText(schema);
	        }
	
	        String procName = profile.getBaseProperties().getProperty(ISybaseJDBCConnectionProfileConstants.PROP_SPNAME); //$NON-NLS-1$
	        if (procName != null && this.mProcNameText != null) {
	        	this.mProcNameText.setText(procName);
	        }
	
        }
        this.connTabComposite.layout();
        setErrorMessage(null);
        
    }

    /**
     * @return Properties
     */
    protected Properties collectProperties() {
        Properties props = new Properties();
        
//        String driverName = EMPTY_STRING;
        String driverClass = EMPTY_STRING;
//        String driverPaths = EMPTY_STRING;
        String driverID = EMPTY_STRING;
        String driverDefnID = EMPTY_STRING;
        String catalog = "%"; //$NON-NLS-1$
        String schema = "%"; //$NON-NLS-1$
        String procName = "%"; //$NON-NLS-1$
        String host = EMPTY_STRING;
        String port = EMPTY_STRING;
        String url = EMPTY_STRING;
        String version = EMPTY_STRING;
        String vendor = EMPTY_STRING;
        
        if (this.combo.getSelectedDriverInstance() != null) {
        	DriverInstance instance = this.combo.getSelectedDriverInstance();
//            driverName = instance.getName();
            driverClass = instance.getNamedProperty("Driver Class"); //$NON-NLS-1$
//            driverPaths = instance.getJarList();
            driverID = instance.getId();
            driverDefnID = getDriverDefnId();
            version = instance.getNamedProperty("Version"); //$NON-NLS-1$
            vendor = instance.getNamedProperty("Vendor"); //$NON-NLS-1$
        }
        if (this.mCatalogText != null && this.mCatalogText.getText() != null) {
        	catalog = this.mCatalogText.getText();
        }
        if (this.mSchemaText != null && this.mSchemaText.getText() != null) {
        	schema = this.mSchemaText.getText();
        }
        if (this.mProcNameText != null && this.mProcNameText.getText() != null) {
        	procName = this.mProcNameText.getText();
        }
        if (this.mURLText != null && this.mURLText.getText() != null) {
        	url = this.mURLText.getText();
        }
        else {
        	url = this.getURL();
        }
        
        props.setProperty(ConnectionProfileConstants.PROP_DRIVER_DEFINITION_ID, driverID);
        if (driverDefnID.endsWith("mssql")) { //$NON-NLS-1$
        	props.setProperty(ISybaseJDBCConnectionProfileConstants.PROP_MULTI_DB_IS_SUPPORTED,
       			"true"); //$NON-NLS-1$
        }
        else if (driverDefnID.indexOf("sybase_ase") > -1 ) { //$NON-NLS-1$
        	props.setProperty(ISybaseJDBCConnectionProfileConstants.PROP_MULTI_DB_IS_SUPPORTED,
       			"true"); //$NON-NLS-1$
        }
        else {
        	props.setProperty(ISybaseJDBCConnectionProfileConstants.PROP_MULTI_DB_IS_SUPPORTED,
				"false"); //$NON-NLS-1$
        }
        props.setProperty(IDBDriverDefinitionConstants.DRIVER_CLASS_PROP_ID, driverClass);
        
        props.setProperty(ISybaseJDBCConnectionProfileConstants.PROP_CATALOG, catalog);
        props.setProperty(ISybaseJDBCConnectionProfileConstants.PROP_SCHEMA, schema);
        props.setProperty(ISybaseJDBCConnectionProfileConstants.PROP_SPNAME, procName);
        props.setProperty(
				IDBConnectionProfileConstants.CONNECTION_PROPERTIES_PROP_ID,
				mShowOptionalTab ? mDBConnProps.getSelection() : "");
        props.setProperty(IDBDriverDefinitionConstants.PASSWORD_PROP_ID, this.mDBPWDText.getText());
        props.setProperty(IDBDriverDefinitionConstants.USERNAME_PROP_ID, this.mDBUIDText.getText());

        props.setProperty(IDBDriverDefinitionConstants.URL_PROP_ID,
        		url);

        props.setProperty(ISybaseJDBCConnectionProfileConstants.PROP_HOST, host);
        props.setProperty(ISybaseJDBCConnectionProfileConstants.PROP_PORT, port);

		props.setProperty(IDBDriverDefinitionConstants.DATABASE_VENDOR_PROP_ID, vendor);		
		props.setProperty(IDBDriverDefinitionConstants.DATABASE_VERSION_PROP_ID, version);		

		return props;
    }
    
    /**
     * @return Properties
     */
    public Properties getProperties() {
    	return collectProperties();
    }
    
    /* (non-Javadoc)
     * @see com.sybase.stf.common.ui.dialogs.IDialogPage#isValid()
     */
    public boolean isValid() {
    	boolean isURLOK = this.mURLText != null && this.mURLText.getText().trim().length() > 0;
    	boolean isUIDOK = this.mDBUIDText != null && this.mDBUIDText.getText().trim().length() > 0;
    	boolean isDriverOK = this.combo.getErrorMessage() == null;
    	if (isURLOK && isUIDOK && isDriverOK) 
    		return true;
    	else {
    		if (!isDriverOK) {
    			this.setErrorMessage(combo.getErrorMessage());
    		}
    		else if (!isURLOK) {
    			this.setErrorMessage(JDBCProfileMessages.getString("JDBCProfileTabs.errors.requiredURL")); //$NON-NLS-1$
    		}
//    		else if (!isUIDOK) {
//    			this.setErrorMessage(JDBCProfileTabs.errors.requiredUID")); //$NON-NLS-1$
//    		}
    	}
    	return false;
    }

	/**
	 * If we changed, fire a changed event.
	 * @param source
	 */
	protected void fireChangedEvent(Object source)
	{
		ChangeEvent e = new ChangeEvent(source);
		// inform any listeners of the resize event
		Object[] listeners = this.changeListeners.getListeners();
		for (int i = 0; i < listeners.length; ++i)
		{
			((ChangeListener) listeners[i]).stateChanged(e);
		}
	}

	/**
	 * Add a change listener
	 * @param listener
	 */
	public void addChangeListener(ChangeListener listener)
	{
		this.changeListeners.add(listener);
	}

	/**
	 * Remove a change listener.
	 * @param listener
	 */
	public void removeChangeListener(ChangeListener listener)
	{
		this.changeListeners.remove(listener);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#dispose()
	 */
	public void dispose() {
		if (this.comboListener != null) {
			this.combo.removeChangeListener(this.comboListener);
		}
		super.dispose();
	}
	
	/**
	 * @return DriverListCombo
	 */
	public DriverListCombo getDriverCombo () {
		return this.combo;
	}
	
	/**
	 * @return
	 */
	public boolean getShowFilterTab() {
		return this.mShowFilterTab;
	}
	/**
	 * @param flag
	 */
	public void setShowFilterTab( boolean flag ) {
		this.mShowFilterTab = flag;
	}
	/**
	 * @return
	 */
	public boolean getShowOptionalTab() {
		return this.mShowOptionalTab;
	}
	/**
	 * @param flag
	 */
	public void setShowOptionalTab( boolean flag ) {
		this.mShowOptionalTab = flag;
	}

    public void setResetPwdFromDriver(boolean b)
    {
        // TODO this empty method is simply to suppress the compilation error
		this._resetPwdFromDriver = b;
    }
}
